#!perl

BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}


use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::Perl::Critic::Subset 3.001.006

use Test::Perl::Critic (-profile => "") x!! -e "";

my $filenames = ['lib/Data/Sah/Coerce/perl/To_array/From_str/expand_glob.pm','lib/Data/Sah/Filter/perl/Path/check_dir_exists.pm','lib/Data/Sah/Filter/perl/Path/check_file_exists.pm','lib/Data/Sah/Filter/perl/Path/check_path_exists.pm','lib/Data/Sah/Filter/perl/Path/check_path_not_exists.pm','lib/Data/Sah/Filter/perl/Path/expand_tilde.pm','lib/Data/Sah/Filter/perl/Path/expand_tilde_when_on_unix.pm','lib/Data/Sah/Filter/perl/Path/strip_slashes.pm','lib/Data/Sah/Filter/perl/Path/strip_slashes_when_on_unix.pm','lib/Sah/Schema/dirname.pm','lib/Sah/Schema/dirname/default_curdir.pm','lib/Sah/Schema/dirname/exists.pm','lib/Sah/Schema/dirname/not_exists.pm','lib/Sah/Schema/dirname/unix.pm','lib/Sah/Schema/dirname/unix/exists.pm','lib/Sah/Schema/dirname/unix/not_exists.pm','lib/Sah/Schema/dirnames/exist.pm','lib/Sah/Schema/filename.pm','lib/Sah/Schema/filename/exists.pm','lib/Sah/Schema/filename/not_exists.pm','lib/Sah/Schema/filename/unix.pm','lib/Sah/Schema/filename/unix/exists.pm','lib/Sah/Schema/filename/unix/not_exists.pm','lib/Sah/Schema/filenames.pm','lib/Sah/Schema/filenames/exist.pm','lib/Sah/Schema/pathname.pm','lib/Sah/Schema/pathname/exists.pm','lib/Sah/Schema/pathname/not_exists.pm','lib/Sah/Schema/pathname/unix.pm','lib/Sah/Schema/pathname/unix/exists.pm','lib/Sah/Schema/pathname/unix/not_exists.pm','lib/Sah/Schema/pathnames.pm','lib/Sah/Schema/pathnames/exist.pm','lib/Sah/SchemaR/dirname.pm','lib/Sah/SchemaR/dirname/default_curdir.pm','lib/Sah/SchemaR/dirname/exists.pm','lib/Sah/SchemaR/dirname/not_exists.pm','lib/Sah/SchemaR/dirname/unix.pm','lib/Sah/SchemaR/dirname/unix/exists.pm','lib/Sah/SchemaR/dirname/unix/not_exists.pm','lib/Sah/SchemaR/dirnames/exist.pm','lib/Sah/SchemaR/filename.pm','lib/Sah/SchemaR/filename/exists.pm','lib/Sah/SchemaR/filename/not_exists.pm','lib/Sah/SchemaR/filename/unix.pm','lib/Sah/SchemaR/filename/unix/exists.pm','lib/Sah/SchemaR/filename/unix/not_exists.pm','lib/Sah/SchemaR/filenames.pm','lib/Sah/SchemaR/filenames/exist.pm','lib/Sah/SchemaR/pathname.pm','lib/Sah/SchemaR/pathname/exists.pm','lib/Sah/SchemaR/pathname/not_exists.pm','lib/Sah/SchemaR/pathname/unix.pm','lib/Sah/SchemaR/pathname/unix/exists.pm','lib/Sah/SchemaR/pathname/unix/not_exists.pm','lib/Sah/SchemaR/pathnames.pm','lib/Sah/SchemaR/pathnames/exist.pm','lib/Sah/Schemas/Path.pm'];
unless ($filenames && @$filenames) {
    $filenames = -d "blib" ? ["blib"] : ["lib"];
}

all_critic_ok(@$filenames);
